package UserAgent::WeightedChoice;
$UserAgent::WeightedChoice::VERSION = '0.01';
=head1 NAME

UserAgent::WeightedChoice - Returns one from the pre-registered UserAgent DB

=head1 SYNOPSIS

    % cat ua.txt
    chrome
    ie
    firefox

    % useragent-weightedchoice-create-db-from-file.pl --catalog_key pc < ua.txt

    % perl -E 'use UserAgent::WeightedChoice; say UserAgent::WeightedChoice->new( catalog_key => "pc" )->choose'
    chrome

=cut

use strict;
use warnings;
use Class::Tiny qw(catalog_key);

use File::Share ();
use List::Util::WeightedChoice ();

sub BUILD {
    my ($self) = @_;

    die "catalog_key required." unless length $self->catalog_key;
}

sub catalog_basename {
    my ($self) = @_;

    File::Share::dist_file  'UserAgent-WeightedChoice', $self->catalog_key;
}

sub choices {
    my ($self) = @_;

    $self->{choices} ||= do {
        my $file = $self->catalog_basename . ".pos";
        open my $in, "<", $file or die $!;
        my @choices = map { chomp; $_ } <$in>;
        \@choices;
    };
}

sub weights {
    my ($self) = @_;

    $self->{weights} ||= do {
        my $file = $self->catalog_basename . ".weights";
        open my $in, "<", $file or die $!;
        my @weights = map { chomp; $_ } <$in>;
        \@weights;
    };
}

sub txt_file_handle {
    my ($self) = @_;

    $self->{txt_file_handle} ||= do {
        my $file = $self->catalog_basename . ".txt";
        open my $in, "<", $file or die $!;
        $in;
    };
}

sub choose {
    my ($self) = @_;

    my $choices = $self->choices;
    my $weights = $self->weights;
    my $pos     = List::Util::WeightedChoice::choose_weighted $choices, $weights;
    my $txt_fh  = $self->txt_file_handle;
    seek $txt_fh, $pos, 0;
    my $choice  = <$txt_fh>;
    chomp $choice;
    $choice;
}

1;
