#!/usr/local/bin/perl

use strict;
use warnings;
use File::Share qw(dist_file);
use Getopt::Long;
use UserAgent::WeightedChoice;

my $opt = {};
GetOptions($opt, "catalog_key=s", "file=s");

die "catalog_key required." unless length $opt->{catalog_key};
die "catalog_key is invalid." if $opt->{catalog_key} =~ m{[\\/:*?"<>|\.]};

my $in = do {
    if ( length $opt->{file} and $opt->{file} ne '-' ) {
        open my $rh, "<", $opt->{file} or die $!;
        $rh;
    }
    else {
        *STDIN;
    }
};

my $seq = 0;
my %line;

while (<$in>) {
    if ( exists $line{$_} ) {
        $line{$_}->{weights}++;
    }
    else {
        $line{$_} = { seq => $seq++, weights => 1 };
    }
}

open my $pos,     ">", dist_file 'UserAgent-WeightedChoice', "$opt->{catalog_key}.pos"     or die $!;
open my $weights, ">", dist_file 'UserAgent-WeightedChoice', "$opt->{catalog_key}.weights" or die $!;
open my $txt,     ">", dist_file 'UserAgent-WeightedChoice', "$opt->{catalog_key}.txt"     or die $!;

for ( sort { $line{$a}{seq} <=> $line{$b}{seq} } keys %line ) {
    $pos    ->print( tell $txt, "\n" );
    $weights->print( $line{$_}{weights}, "\n" );
    $txt    ->print( $_ );
}
